#---	This file plots the priors of lasso penalty and enet penalty, giving an intuition on how the 
#	values of lambda1 and lambda2 change the shape of the prior, and thus the influence on the 
#	variable selection.

	prior = function(x) {
		temp = (lambda2/(2*sigma2))^(0.5) * exp(-lambda1^2/(8*sigma2*lambda2)) * exp(-1/(2*sigma2)*(lambda1*abs(x)+lambda2*x^2))
		temp = temp * (pgamma((lambda1^2)/(8*lambda2*sigma2), 0.5, lower=FALSE) * gamma(0.5))^(-1)
		return(temp)
	}

	prior.lasso = function(x) {
		temp = lambda1/(4*sigma2) * exp(-lambda1*abs(x)/(2*sigma2))
		return(temp)
	}

	prior.sqrt = function(x) {
		temp = exp(-sqrt(abs(x)))/(4*gamma(2))
		return(temp)
	}

	sigma2 = 1

	par(mfrow=c(1,2))

#--- center

	x = seq(-2,2,length=501)

	lambda1 = 5
	lambda2 = 0
	plot(	x,prior.lasso(x),
		type = "l", ylim = c(0,1.25), lty = "solid", 
		main = expression(paste("Priors with different ", lambda[1], "and ", lambda[2], sep="")),
		xlab = expression(beta),
		ylab = "density function",
		lwd=2)
	lambda1 = 1
	lambda2 = 5
	lines(x,prior(x), lty="dashed", lwd=2)
	lambda1 = 0
	lambda2 = 5
	lines(x,prior(x), lty="dotted", lwd=2)

	legend(locator(n=1),
	legend = c(expression(paste(lambda[1],"=5 ",lambda[2],"=0")),
		expression(paste(lambda[1],"=1 ",lambda[2],"=5")),
		expression(paste(lambda[1],"=0 ",lambda[2],"=5"))),
		lty=c("solid","dashed","dotted"), lwd=c(2,2,2),bty="n")

#--- tail

	x = seq(1,3,length=501)

	lambda1 = 5
	lambda2 = 0
	plot(	x,prior.lasso(x),
		type = "l", ylim = c(0,0.12), lty = "solid", 
		main = expression(paste("Tail behaviors with different ", lambda[1], "and ", lambda[2], sep="")),
		xlab = expression(beta),
		ylab = "density function",
		lwd=2)
	lambda1 = 1
	lambda2 = 5
	lines(x,prior(x), lty="dashed", lwd=2)
	lambda1 = 0
	lambda2 = 5
	lines(x,prior(x), lty="dotted", lwd=2)

	legend(locator(n=1),
	legend = c(expression(paste(lambda[1],"=5 ",lambda[2],"=0")),
		expression(paste(lambda[1],"=1 ",lambda[2],"=5")),
		expression(paste(lambda[1],"=0 ",lambda[2],"=5"))),
		lty=c("solid","dashed","dotted"), lwd=c(2,2,2),bty="n")

#---	Sqrt prior

	x = seq(0,5,length=500)

	plot(x,grad(prior,x), type="l")
	lines(x,grad(prior.lasso,x), lty="dotted")
	lines(x,grad(prior.sqrt,x), lty="dashed")


