	data(diabetes)
	x = diabetes$x
	y = diabetes$y

	#---	Using BL

	lambda.ini = GSIN(scale(x),y-mean(y))
	result.bl = GSL_BL(x,y,lambda1=lambda.ini$lambda1,tol=0.01,itermax=15)
	
	#---	Using BEN

	lambda.ini = GSIN(scale(x),y-mean(y))
	result.ben = GSL(x,y,lambda1=lambda.ini$lambda1,lambda2=lambda.ini$lambda2)

	#---	Using ENET

	result.en = ENL(scale(x),y-mean(y),lambda2 = c(0,0.01,0.1,1,10,100,1000))

	#---	Using LARS

	result.lars = LARSL(scale(x),y-mean(y))
	result.lars$s

	#---	Using Least Square

	result.ls = lm(y-mean(y) ~ 0 + scale(x))

	save.image(file = "BENdiabetes.RData")
	
#---	2.1. Load the workspace

	load("dia and pro\\BENdiabetes.RData")

#---	2.2. Some plots

	for (i in 1:10) {
		if((i-1)/4 == as.integer(i/4)) {
			x11()
			par(mfrow = c(2,2))
		}

		temp = result.bl$beta[,i][seq(1,10000,by=10)]
		plot(density(temp),main=i)
		lines(sort(temp),dnorm(sort(temp),mean=mean(temp), sd=sqrt(var(temp))),col="red")
		temp.line = quantile(temp, probs = c(0.025, 0.975))
		abline(v = temp.line)

		#plot(temp, type="l")
		#acf(temp)
		#hist(temp)
	}
	
	plot(seq(-40,40,length=10), 1:10, ylim=rev(range(1:10)), axes=F, type="n", xlab="Coefficients", ylab="Variable number")
	axis(1, at = seq(-40,40,20))
	axis(2, at = seq(1, 10, 1))
	box()
	abline(v=0)
	temp1 = apply(result.ben$beta, 2, median)
	temp2 = apply(result.bl$beta,2,median)
	for (i in 1:10) {
		points(result.ls$coef[i],i,pch=4)
		points(temp1[i], i, pch=3)
		points(temp2[i], i, pch=1)
		points(result.en$beta[i],i+0.15,pch=2)
		points(result.lars$beta[i],i-0.15,pch=6)
	}
	#legend(locator(n=1),legend=c("BEN","BL","EN", "lasso", "OLS"), pch=c(3, 1, 2, 6, 4))

#---	2.3. Variable selection

	par(mar=c(5,4,2,2))
	par(mfrow=c(1,2))
	level = 0.5
	method = 2
	vss.ben = NULL
	for (d in seq(0,6*(method==2)+(method==3),by=0.1)) {
		intervals = NULL
		interval.prob = NULL
		for (i in 1:10) {
			if (method == 1) intervals = rbind(intervals, quantile(result.ben$beta[,i], probs = c(level/2, 1-level/2)))
			if (method == 2) intervals = rbind(intervals, c(-1,1)*d)
			if (method == 3) intervals = rbind(intervals, c(-1,1) * sqrt(var(result.ben$beta[,i])))
		}
		for (i in 1:10) {
			temp = ecdf(result.ben$beta[,i])
			interval.prob = c(interval.prob, temp(intervals[i,2]) - temp(intervals[i,1]))
		}
		#	names = c("age","sex","bmi","map","tc","ldl","hdl","tch","ltg","glu") 
		#	vs.ben = data.frame(names, intervals, interval.prob)
		#	temp = order(vs.ben[,4])
		#	vs.ben = vs.ben[temp,]

		if (method == 2) vss.ben = rbind(vss.ben, interval.prob)
		if (method == 3) vss.ben = rbind(vss.ben, (interval.prob < d))
	}
	
	names = c("age","sex","bmi","map","tc","ldl","hdl","tch","ltg","glu") 
	if (method == 2) { xlab="Radius of the interval"
			   ylab="Coverage probability"
			   main="Variable selection for different interval radii, BEN"} 
	if (method == 3) { xlab="Probability threshold"
			   ylab="Variable selection"
			   main="Variable selection for different thresholds, BEN"} 
	plot(seq(0,6*(method==2)+(method==3),by=0.1), seq(0,1,length=length(seq(0,6*(method==2)+(method==3),by=0.1))), type = "n", ylab = ylab, xlab=xlab)
	for (i in 1:10) {
		if (method == 2) temp = vss.ben[length(seq(0,6,by=0.1)),i]
		if (method == 3) temp = i*0.1
		lines(seq(0,6*(method==2)+(method==3),by=0.1), (vss.ben[,i] != 0) * i * 0.1 * (method==3) + vss.ben[,i] * (method ==2))
		text(x=5.5*(method==2)+0.9*(method==3), y=temp, labels=names[i], cex=0.8)
	}

	vss.bl = NULL
	for (d in seq(0,6*(method==2)+(method==3),by=0.1)) {
		intervals = NULL
		interval.prob = NULL
		for (i in 1:10) {
			if (method == 1) intervals = rbind(intervals, quantile(result.bl$beta[,i], probs = c(level/2, 1-level/2)))
			if (method == 2) intervals = rbind(intervals, c(-1,1)*d)
			if (method == 3) intervals = rbind(intervals, c(-1,1) * sqrt(var(result.bl$beta[,i])))
		}
		for (i in 1:10) {
			temp = ecdf(result.bl$beta[,i])
			interval.prob = c(interval.prob, temp(intervals[i,2]) - temp(intervals[i,1]))
		}
		#	names = c("age","sex","bmi","map","tc","ldl","hdl","tch","ltg","glu") 
		#	vs.bl = data.frame(names, intervals, interval.prob)
		#	temp = order(vs.bl[,4])
		#	vs.bl = vs.bl[temp,]

		if (method == 2) vss.bl = rbind(vss.bl, interval.prob)
		if (method == 3) vss.bl = rbind(vss.bl, (interval.prob < d))
	}
	
	names = c("age","sex","bmi","map","tc","ldl","hdl","tch","ltg","glu") 
	if (method == 2) { xlab="Radius of the interval"
			   ylab="Coverage probability"
			   main="Variable selection for different interval radii, BEN"} 
	if (method == 3) { xlab="Probability threshold"
			   ylab="Variable selection"
			   main="Variable selection for different thresholds, BEN"} 
	plot(seq(0,6*(method==2)+(method==3),by=0.1), seq(0,1,length=length(seq(0,6*(method==2)+(method==3),by=0.1))), type = "n", ylab = ylab, xlab=xlab)
	for (i in 1:10) {
		if (method == 2) temp = vss.bl[length(seq(0,6,by=0.1)),i]
		if (method == 3) temp = i*0.1
		lines(seq(0,6*(method==2)+(method==3),by=0.1), (vss.bl[,i] != 0) * i * 0.1 * (method==3) + vss.bl[,i] * (method ==2))
		text(x=5.5*(method==2)+0.9*(method==3), y=temp, labels=names[i], cex=0.8)
	}
