	phi = function(t) {
		
		#	The function to be used in GSL

		return(t^(-1/2)*exp(-t))
	}

	my.rgig = function (n, Theta) {
		
		#	The function to generate Generalized Inverse Gaussian random variables. This is 
		#	the modified version to take care of the extremely small psi.

		lambda <- Theta[1]
		chi <- Theta[2]
		psi <- Theta[3]
		if (chi <= 0) 
		stop("chi must be positive")
		if (psi <= 0) 
		stop("psi must be positive")
		if (lambda == 1) {
		stop(return(rgig1(n, c(chi, psi))))
		}
		if (psi*chi <= 1e-13) {
		#	As extremely small psi*chi would give infinite number of rejections for the algorithm below, 
		#	we are trying to fix it by drawing a uniform random variable. But the problem is that only 
		#	small psi would make dgig look like a uniform density. But it is the small psi*chi that
		#	is causing the problem. What should we do if psi is not too small but chi is?

		stop(return(runif(n, min=0, max=1e6)))
		}
		alpha <- sqrt(psi/chi)
		beta <- sqrt(psi * chi)
		m <- (lambda - 1 + sqrt((lambda - 1)^2 + beta^2))/beta
		g <- function(y) {
		0.5 * beta * y^3 - y^2 * (0.5 * beta * m + lambda + 1) + 
		    y * ((lambda - 1) * m - 0.5 * beta) + 0.5 * beta * 
		    m
		}
		upper <- m
		while (g(upper) <= 0) upper <- 2 * upper
		yM <- uniroot(g, interval = c(0, m), tol = min(.Machine$double.eps^0.25, 
		(.Machine$double.eps + g(0)/10)))$root
		yP <- uniroot(g, interval = c(m, upper))$root
		a <- (yP - m) * (yP/m)^(0.5 * (lambda - 1)) * exp(-0.25 * 
		beta * (yP + 1/yP - m - 1/m))
		b <- (yM - m) * (yM/m)^(0.5 * (lambda - 1)) * exp(-0.25 * 
		beta * (yM + 1/yM - m - 1/m))
		c <- -0.25 * beta * (m + 1/m) + 0.5 * (lambda - 1) * log(m)
		output <- numeric(n)
		for (i in 1:n) {
		needValue <- TRUE
		while (needValue == TRUE) {
		    R1 <- runif(1)
		    R2 <- runif(1)
		    Y <- m + a * R2/R1 + b * (1 - R2)/R1
		    if (Y > 0) {
			if (-log(R1) >= -0.5 * (lambda - 1) * log(Y) + 
			  0.25 * beta * (Y + 1/Y) + c) {
			  needValue <- FALSE
			}
		    }
		}
		output[i] <- Y
		}
		output/alpha
		}

	my.rgig.new = function(n=1, Theta) {
		
		#	I can only handle n=1 for the moment.
		#	This function does not work very well. When the parameters are not good,
		#	this would run into infinite rejections.

		lambda = Theta[1]
		chi = Theta[2]
		psi = Theta[3]
		flag = 1
		while (flag) {
			t = rgamma(n, shape=lambda, scale=2/psi)
			y = rexp(n, rate=1/t)
			flag = (y < chi/2)
		}
		return(t)
	}

	GS = function(x,y,lambda1,lambda2,n.burnin = 1000,n.sampler=11000) {

		#	This is the Gibbs sampler for BEN. In this function, we are not updating lambda1
		#	and lambda2. 

		#	n.burnin is the length of the burn-in period
		#	n.sampler is the length of the Markov chain
		#	n.lambda is the number of iterations we want for updating lambda

		x = scale(x)
		y = y - mean(y)
		n = dim(x)[1]
		p = dim(x)[2]

		#	The parameters with ".c" are the temporary ones that we use for updating.
		#	The parameters with ".p" are the recorded ones.

		#---	Initialization
		beta.c = rep(1, p)
		sigma2.c = 10000
		tau.c = rep(1.5, p)

		#---	Iteration
		beta.p = sigma2.p = tau.p = NULL
		xtx = t(x) %*% x
		ytx = t(y) %*% x
		for (iter in 1:n.sampler) {
			if (iter/1000 == as.integer(iter/1000)) {print(iter)}
		
			a.temp = lambda2 * tau.c / (tau.c - 1) + diag(xtx)
			for (j in 1:p) {
				bj.temp = sum((xtx)[-j,j] * beta.c[-j]) - 2 * ytx[j]
				beta.c[j] = rnorm(1, mean=-bj.temp/(2*a.temp[j]), sd=sqrt(sigma2.c/a.temp[j]))
			}
			
			temp.nu = sqrt(lambda1) / (2*lambda2*abs(beta.c))
			for (j in 1:p) {
				flag = 1
				while (flag) {
					temp = rinvGauss(n=1, nu=temp.nu[j], lambda=lambda1/(4*lambda2*sigma2.c))
					flag = (temp <= 0)
				}
				tau.c[j] = 1 + 1 / temp
			}

			a.temp = n/2 + p
			b.temp = 1/2*sum( (y - x%*%beta.c)^2 ) + 1/2*lambda2*sum((tau.c / (tau.c - 1)) * beta.c^2) + 1/8*lambda1^2/lambda2*sum(tau.c)
			#--- The MH algorithm
			#	Z.temp = rinvgamma(n=1, shape = a.temp, scale = b.temp)
			#	prob.temp = min(1, (gamma_inc(1/2, lambda1^2/(8*sigma2.c*lambda2)) / gamma_inc(1/2, lambda1^2/(8*Z.temp*lambda2)))^p)
			#	u.temp = runif(1)
			#	if (u.temp <= prob.temp) {sigma2.c = Z.temp} 

			#--- The AR algorithm
			flag.temp = 1
			while(flag.temp) {
				z.temp = rinvgamma(n=1, shape = a.temp, scale = b.temp)
				u.temp = runif(1)
				if (log(u.temp) <= p*log(gamma(0.5))-p*log(gamma_inc(1/2, lambda1^2/(8*z.temp*lambda2)))) {
					sigma2.c = z.temp
					flag.temp = 0
				}
			}

			beta.p = rbind(beta.p, beta.c)
			tau.p = rbind(tau.p, tau.c)
			sigma2.p = c(sigma2.p, sigma2.c)
		}
		
		result = list(beta = beta.p[-(1:n.burnin),], sigma2 = sigma2.p[-(1:n.burnin)], tau = tau.p[-(1:n.burnin),])
		return(result)
	}
	
	GSL = function(x,y,lambda1,lambda2,n.burnin=1000,n.sampler=11000,itermax=10,tol=0.1) {

		#	This function is based on the function GS. Here we can update the penalty parameters
		#	lambda1 and lambda2. This is done by EM. The result contains all lambda's we have
		#	tried and the Markov Chain using the last lambda.

		#	itermax: The maximum number of EM iterations
		#	tol: The tolerance criterion for convergence

		n = dim(x)[1]
		p = dim(x)[2]
		flag = 1
		counter = 1
		lambda.record = c(lambda1, lambda2)
		f.record = NULL

		while(flag) {
			print(counter)
			print(paste("f.record=",f.record))
			print(lambda.record)
			result = GS(x,y,lambda1,lambda2,n.burnin,n.sampler)

			f = function(a) {
				a1 = a[1]
				a2 = a[2]
				temp1 = p*log(a1) - p*mean(gamma_inc(1/2, a1^2/(8*result$sigma2*a2)))
				temp2 = 0
				for (j in 1:p) {
					temp2 = temp2 + mean(result$tau[,j] / (result$tau[,j]-1) * result$beta[,j]^2 / result$sigma2)
				}
				temp2 = -a2/2 * temp2
				temp3 = 0
				for (j in 1:p) {
					temp3 = temp3 + mean(result$tau[,j] / result$sigma2)
				}
				temp3 = -a1^2/(8*a2) * temp3
				return(- temp1 - temp2 - temp3)
			}

			df = function(a) {
				a1 = a[1]
				a2 = a[2]
				temp1 = p/a1 + p*a1/(4*a2)*mean(gamma_inc(1/2, a1^2/(8*result$sigma2*a2))^(-1) * phi(a1^2/(8*result$sigma2*a2)) / result$sigma2)
				temp2 = 0
				for (j in 1:p) {
					temp2 = temp2 + mean(result$tau[,j] / result$sigma2)
				}
				temp2 = -a1/(4*a2) * temp2
				result.1 = temp1 + temp2

				temp1 = -p*a1^2/(8*a2^2)*mean(gamma_inc(1/2, a1^2/(8*result$sigma2*a2))^(-1) * phi(a1^2/(8*result$sigma2*a2)) / result$sigma2)
				temp2 = 0
				for (j in 1:p) {
					temp2 = temp2 + mean(result$tau[,j] / (result$tau[,j]-1) * result$beta[,j]^2 / result$sigma2)
				}
				temp2 = -1/2 * temp2
				temp3 = 0
				for (j in 1:p) {
					temp3 = temp3 + mean(result$tau[,j] / result$sigma2)
				}
				temp3 = a1^2/(8*a2^2) * temp3
				result.2 = temp1 + temp2 + temp3
				return(c(-result.1, -result.2))
			}
			
			lambda.current = c(lambda1, lambda2)
			lambda.update = constrOptim(lambda.current, f, grad=df, ui=diag(c(1,1)), ci=.Machine$double.eps*c(1,1))$par
			# lambda.update = constrOptim(lambda.current, f, grad=NULL,ui=diag(c(1,1)), ci=.Machine$double.eps*c(1,1))$par
			flag = (counter < itermax) & ( sum((lambda.current - lambda.update)^2) >= tol )
			convergence = ( sum((lambda.current - lambda.update)^2) < tol )
			lambda.record = rbind(lambda.record, lambda.update)
			f.current = f(lambda.current)
			f.record = c(f.record, f.current)
			lambda1 = lambda.update[1]
			lambda2 = lambda.update[2]
			counter = counter + 1
		}
		out = list(beta = result$beta, sigma2 = result$sigma2, tau = result$tau, lambda = lambda.record, convergence = convergence, f=f.record)
		return(out)
	}

	GSIN = function(x, y) {
		
		#	This function picks the initial value of lambda1 and lambda2 to be used later in
		#	the EM algorithm in GSL.

		x = scale(x)
		y = y - mean(y)
		n = dim(x)[1]
		p = dim(x)[2]

		ls = lm(y ~ 0 + x)
		sigma2hat = var(ls$resid)
		betahat = na.omit(ls$coef)
		
		result = list(lambda1 = p*sqrt(sigma2hat)/sum(abs(betahat)), lambda2 = 0.0001)
		return(result)
	}

	ENL = function(x,y,lambda2 = c(0,0.01,0.1,1,10,100,1000), K = 10, mode = "fraction", max.steps = 200) {

		#	The elastic net algorithm with L1 penalty lambda1 chosen by K-fold cross validation.
		#	The loss function is MSE. The default CV is on the fractions seq(0,1,length=100). This 
		#	is easy to control. If we want to use lambda1, it is not straightforward how to
		#	decide the abscissa. The L2 penalty term lambda2 is chosen as the best one in the pool.

		n = dim(x)[1]
		p = dim(x)[2]
		s = err = NULL	# to record, for each lambda2, the L1 norm fraction chosen by CV and the 
				# corresponding prediction error.
		if (mode == "fraction") 
			abscissa = seq(0,1,length=100)
		if (mode == "step")
			abscissa = 1:max.steps
		for (index2 in 1:length(lambda2)) {	
			#	The K-fold cross validation
			temp = cv.enet(x,y,lambda=lambda2[index2],s=abscissa,mode=mode,K=K, plot.it=F)
			s = c(s, abscissa[which(temp$cv == min(temp$cv))])
			err = c(err, min(temp$cv))
		}
		lambda2.chosen = lambda2[which(err == min(err))]
		s.chosen = s[which(err == min(err))]

		temp = enet(x,y,lambda=lambda2.chosen, max.steps = max.steps)
		result.en = predict(temp, s = s.chosen, type = "coefficients", mode = mode)
		result = list(s = s.chosen, lambda2 = lambda2.chosen, beta = result.en$coef, err = min(err))
		return(result)
	}

	ENLV = function(x.train, y.train, x.validation, y.validation,lambda2 = c(0,0.01,0.1,1,10,100,1000), mode = "fraction", max.steps = 200) {

		#	The elastic net algorithm with L1 penalty lambda1 chosen by the validation data set.
		#	The loss function is MSE. The default abscissa is on the fractions seq(0,1,length=100). This 
		#	is easy to control. If we want to use lambda1, it is not straightforward how to
		#	decide the abscissa. The L2 penalty term lambda2 is chosen as the best one in the pool.

		n.train = dim(x.train)[1]
		n.validation = dim(x.validation)[1]
		p = dim(x.train)[2]
		s = err = NULL	# to record, for each lambda2, the L1 norm fraction chosen by the validation set 
				# and the corresponding prediction error.
		if (mode == "fraction") 
			abscissa = seq(0,1,length=100)
		if (mode == "step")
			abscissa = 1:max.steps
		for (index2 in 1:length(lambda2)) {
			temp = enet(x.train, y.train, lambda=lambda2[index2])
			pred.temp = predict(temp, x.validation, type="fit", s=abscissa,mode=mode)$fit
			mse = apply((as.numeric(y.validation) - pred.temp)^2, 2, mean)
			s = c(s, abscissa[which(mse == min(mse))])
			err = c(err, min(mse))
		}
		lambda2.chosen = lambda2[which(err == min(err))[1]]
		s.chosen = s[which(err == min(err))[1]]

		temp = enet(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)),lambda=lambda2.chosen, max.steps = max.steps)
		if (lambda2.chosen == 0) temp = lars(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)), max.steps = max.steps)
		result.en = predict(temp, s = s.chosen, type = "coefficients", mode = mode)
		result = list(s = s.chosen, lambda2 = lambda2.chosen, beta = result.en$coef, err = min(err))
		return(result)
	}

	LARSL = function(x,y,K = 10) {
		abscissa = seq(0,1,length=100)
		temp = cv.lars(x,y,fraction=abscissa,K=K, plot.it=F)
		s = abscissa[which(temp$cv == min(temp$cv))]
		err = min(temp$cv)

		temp = lars(x,y)
		result.lars = predict(temp, s = s, type = "coefficients", mode = "fraction")
		result = list(s = s, beta = result.lars$coef, err = err)
		return(result)
	}

	LARSLV = function(x.train, y.train, x.validation, y.validation) {
		abscissa = seq(0,1,length=100)
		temp = lars(x.train,y.train)
		pred.temp = predict(temp, x.validation, type="fit", s=abscissa,mode="fraction")$fit
		mse = apply((as.numeric(y.validation) - pred.temp)^2, 2, mean)
		s = abscissa[which(mse == min(mse))]
		err = min(mse)

		temp = lars(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)))
		result.lars = predict(temp, s = s, type = "coefficients", mode = "fraction")
		result = list(s = s, beta = result.lars$coef, err = err)
		return(result)
	}

	GS_BL = function(x,y,lambda1,n.burnin = 1000,n.sampler=11000) {

		#	This is the Gibbs sampler for BL. In this function, we are not updating lambda1.

		#	n.burnin is the length of the burn-in period
		#	n.sampler is the length of the Markov chain
		#	n.lambda is the number of iterations we want for updating lambda

		x = scale(x)
		y = y - mean(y)
		n = dim(x)[1]
		p = dim(x)[2]

		#	The parameters with ".c" are the temporary ones that we use for updating.
		#	The parameters with ".p" are the recorded ones.

		#---	Initialization
		beta.c = rep(1, p)
		sigma2.c = 1
		tau2.c = rep(1.5, p)
		iD = diag(1/tau2.c)

		#---	Iteration
		beta.p = sigma2.p = tau2.p = NULL
		xtx = t(x) %*% x
		xty = t(x) %*% y

		for (iter in 1:n.sampler) {
			if (iter/1000 == as.integer(iter/1000)) {print(iter)}
		
			A = xtx + iD
			iA = solve(A)
			beta.c = mvrnorm(n=1, mu = iA%*%xty, Sigma=sigma2.c*iA)

			temp.nu = lambda1*sqrt(sigma2.c) / abs(beta.c)
			for (j in 1:p) {
				flag = 1
				while (flag) {
					temp = rinvGauss(n=1, nu=temp.nu[j], lambda=lambda1^2)
					flag = (temp <= 0)
				}
				tau2.c[j] = 1 / temp
			}
			iD = diag(1/tau2.c)

			temp =  1/2 * ( sum(y^2) - 2*sum(xty*beta.c) + sum(beta.c*(A%*%beta.c)) )
			sigma2.c = rinvgamma(n=1, shape = (n+p)/2, scale = temp)

			beta.p = rbind(beta.p, beta.c)
			tau2.p = rbind(tau2.p, tau2.c)
			sigma2.p = c(sigma2.p, sigma2.c)
		}
		
		result = list(beta = beta.p[-(1:n.burnin),], sigma2 = sigma2.p[-(1:n.burnin)], tau2 = tau2.p[-(1:n.burnin),])
		return(result)
	}

	GSL_BL = function(x,y,lambda1,n.burnin=1000,n.sampler=11000,itermax=10,tol=0.05) {

		n = dim(x)[1]
		p = dim(x)[2]
		flag = 1
		counter = 1
		lambda.record = lambda1

		while(flag) {
			print(counter)
			print(lambda.record)
			result = GS_BL(x,y,lambda1,n.burnin,n.sampler)
			
			lambda.current = lambda1
			lambda.update = sqrt(2*p / (sum(apply(result$tau2, 2, mean))))
			
			flag = (counter < itermax) & ( sum((lambda.current - lambda.update)^2) >= tol )
			convergence = ( sum((lambda.current - lambda.update)^2) < tol )
			lambda.record = c(lambda.record, lambda.update)
			lambda1 = lambda.update
			counter = counter + 1
		}
		out = list(beta = result$beta, sigma2 = result$sigma2, tau2 = result$tau2, lambda = lambda.record, convergence = convergence)
		return(out)
	}

