library(MCMCpack)

set.seed(12356)
n.train = 20			# number of training replicates
p = 8				# number of parameters
beta.true = c(3, 1.5, 0, 0, 2, 0, 0, 0)
sigma.true = 3
x.sigma = matrix(NA, p, p)
rho = 0.5
for(i in 1:p) {
	for (j in 1:p) {
		x.sigma[i,j] = rho^abs(i-j)
	}
}
x.train = mvrnorm(n=n.train, mu=rep(0,p), Sigma=x.sigma, empirical=T)		# This x is standardized.
y.train = x.train %*% beta.true + sigma.true * rnorm(n.train)
xty = t(x.train)%*%y.train
xtx = t(x.train)%*%x.train

f = function(lambda1, lambda2) {
	mci=20
	# mci stands for "Monte Carlo Integration" and means how many random numbers do we use to 
	# approximate the integral

	value = NULL
	a = 0.1
	b = 0.1
	set.seed(1235)
	sigma2 = pmax(pmin(rinvgamma(mci, shape=a, scale=b), 1e10), 2)
	for (nsigma in 1:mci) {
		tau = NULL
		for (j in 1:p) {
			scale.temp = pmin(8*lambda2*sigma2[nsigma]/lambda1^2, 1e10)
			tau.temp = rgamma(n = mci, shape = 0.5, scale = scale.temp)
			flag = any (tau.temp <= 1)
			count = 0
			while(flag) {
				count = count + 1
				tau.temp[which(tau.temp<=1)] = rgamma(n = length(which(tau.temp<=1)), shape = 0.5, scale = scale.temp)
				flag = any (tau.temp <= 1) & count <= 20
			}
			tau.temp[which(tau.temp<=1)] = runif(sum(tau.temp<=1),min=1.1,max=100000)
			tau = cbind(tau,tau.temp)
		}
		for (ntau in 1:mci) {
			A = xtx + lambda2*diag(tau[ntau,]/(tau[ntau,]-1))
			iA = solve(A)
			dA = det(A)

			value = c(value, 
			(sigma2[nsigma])^(-n.train/2) * 
			prod(sqrt(tau[ntau,]/(tau[ntau,]-1))) * 
			lambda2^(p/2) * 
			exp(-1/(2*sigma2[nsigma]) * (sum(y.train^2)-t(xty)%*%iA%*%xty)) / sqrt(dA))
		}
	}
	return(mean(value))
}
	
lambda1 <- seq(0.01, 10, length= 30)
lambda2 <- seq(0.01, 100, length= 30)
z = matrix(NA, 30,30)
for (i in 1:30) {
	for (j in 1:30) {
		print(j)
		z[i,j] = f(lambda1[i],lambda2[j])
	}
}

	
