library(MASS)
library(MCMCpack)
library(SuppDists)
library(TeachingDemos)
library(lars)
library(elasticnet)
library(boot)
library(HyperbolicDist)
library(gsl)
library(lasso2)
library(golubEsets)
library(Biobase)
library(QRMlib)

setwd("D:\\Files\\A-Study\\Research\\BEN\\code\\")
# setwd("H:\\BEN 2 times\\")
set.seed(12345)

#--------------------------------------------------------------------------------------------------
#	1.	the Gibbs Sampler function and other functions
#--------------------------------------------------------------------------------------------------

	source("functions.R")

#--------------------------------------------------------------------------------------------------
#	2.	The diabetes data
#--------------------------------------------------------------------------------------------------

	source("diabetes.R")

#--------------------------------------------------------------------------------------------------
#	3.	The prostate cancer data
#--------------------------------------------------------------------------------------------------

	source("prostate.R")

#--------------------------------------------------------------------------------------------------
#	4.	The Leukemia data
#--------------------------------------------------------------------------------------------------

	source("leukemia.R")

#--------------------------------------------------------------------------------------------------
#	5.	Simulation 
#--------------------------------------------------------------------------------------------------


#---	5.1 Simulation

if(0) {
	for (sim.i in c(1,2,4,5)) {
		source("xgen.R")	
		for (sim in 1:n.sim) {
			source("datagen.R")
			save(x.train, x.validation, x.test, x, y.train, y.validation, y.test, y, file = paste("sim data\\sim_", sim.i, "\\sim data ", sim, ".Rdata", sep=""))
		}
	}
}
	set.seed(12345)

#---	Using BEN

	for (sim.i in 5) {
		beta.ben = lambda.ben = y.ben = NULL
		for (sim in 1:n.sim) {
			print(paste("This is simulation number ", sim, sep=""))
			load(paste("sim data\\sim_", sim.i, "\\sim data ", sim, ".Rdata", sep=""))

			lambda.ini = GSIN(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)))
			result.ben = GSL(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)),lambda1=lambda.ini$lambda1,lambda2=1, itermax=20, tol = 0.05)
			#result.ben = GSL(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)),lambda1=1,lambda2=70, itermax=20, tol = 0.05)

			beta.ben = rbind(beta.ben, cbind(result.ben$beta, sim = sim))
			lambda.ben = rbind(lambda.ben, result.ben$lambda[dim(result.ben$lambda)[1],])
			y.ben = rbind(y.ben, cbind(y, sim = sim))

			if (sim %in% (10*(1:5))) {
				save(beta.ben, lambda.ben, y.ben, x, file = paste("sim data\\sim_",sim.i,"\\ben.Rdata", sep=""))
			}
		}
	}

#---	Using BL

	for (sim.i in 5) {
		beta.bl = lambda.bl = y.bl = NULL
		for (sim in 1:n.sim) {
			load(paste("sim data\\sim_", sim.i, "\\sim data ", sim, ".Rdata", sep=""))
			print(paste("This is simulation number ", sim, sep=""))

			lambda.ini = GSIN(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)))
			result.bl = GSL_BL(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)),lambda1=lambda.ini$lambda1,itermax=13, tol=0.01)
			#result.bl = GSL_BL(scale(rbind(x.train, x.validation)),c(y.train, y.validation) - mean(c(y.train,y.validation)),lambda1=8.585620,itermax=13, tol=0.01)
			beta.bl = rbind(beta.bl, cbind(result.bl$beta, sim = sim))
			lambda.bl = c(lambda.bl, result.bl$lambda[length(result.bl$lambda)])
			y.bl = rbind(y.bl, cbind(y, sim = sim))

			if (sim %in% (10*(1:5))) {
				save(beta.bl, lambda.bl, y.bl, x, file = paste("sim data\\sim_", sim.i, "\\bl.Rdata", sep=""))
			}
		}
	}

#---	Using enet

	set.seed(12345)
	for (sim.i in 4) {		
		beta.en = lambda.en = resid.en = NULL
		for (sim in 1:n.sim) {
			load(paste("sim data\\sim_", sim.i, "\\sim data ", sim, ".Rdata", sep=""))

			result.en = ENLV(x.train, y.train, x.validation, y.validation)

			beta.en = rbind(beta.en, result.en$beta)
			lambda.en = rbind(lambda.en, c(result.en$s, result.en$lambda2))
			resid.en = c(resid.en, mean(((y.test-mean(y.test))  - scale(x.test) %*% result.en$beta)^2))
		}
		save(beta.en, lambda.en, resid.en, file=paste("sim data\\sim_", sim.i, "\\en.Rdata", sep=""))
	}

#---	Using lars

	set.seed(12345)
	for (sim.i in 4) {		
		beta.lars = lambda.lars = resid.lars = NULL
		for (sim in 1:n.sim) {
			load(paste("sim data\\sim_", sim.i, "\\sim data ", sim, ".Rdata", sep=""))
			result.lars = LARSLV(x.train, y.train, x.validation, y.validation)

			beta.lars = rbind(beta.lars, result.lars$beta)
			lambda.lars = c(lambda.lars, result.lars$s)
			resid.lars = c(resid.lars, mean(((y.test - mean(y.test)) - scale(x.test) %*% result.lars$beta)^2))
		}
		save(beta.lars, lambda.lars, resid.lars, file=paste("sim data\\sim_", sim.i, "\\lars.Rdata", sep=""))
	}


#--------------------------------------------------------------------------------------------------


#---	5.2 Result processing

	sim.i = 5	# This works for "result process.R"
	method = 3	# The method for selecting variables for the BEN and BL method. Method = 1 stands for
			# credible interval criterion. Method = 3 stands for the scaled neighborhood criterion.
			# See the paper for details.
	d = 0.5
	source("result process.R")

#---	5.3 Summary

	temp = rbind(c(median(resid.ben), sd.ben, apply(bsm.ben, 2, sum)), 
		     c(median(resid.en), sd.en, apply(beta.en == 0, 2, sum)), 
		     c(median(resid.bl), sd.bl, apply(bsm.bl, 2, sum)), 
		     c(median(resid.lars), sd.lars, apply(beta.lars == 0, 2, sum)))
	result.sim = data.frame(temp, row.names = c("BEN", "EN", "BL", "LASSO"))
	temp = NULL
	for (i in 1:p) temp = c(temp, paste("beta",i,sep=""))
	names(result.sim) = c("median of MSE", "sd of MedianSE", temp)

#---	5.4 The ROC curve

	sim.i = 5
	length = 20
	error = NULL
	ind1 = which(beta.true != 0)
	ind0 = which(beta.true == 0)

	load(paste("sim data\\sim_", sim.i, "\\en and lars\\en.Rdata", sep=""))
	beta.en = beta.en[1:50,]
	if (length(ind0)==0) {type1.en=0} else {type1.en=(1 - apply(beta.en[,ind0] == 0, 1, sum) / length(ind0))}
	type2.en = apply(beta.en[,ind1] == 0, 1, sum) / length(ind1)

	load(paste("sim data\\sim_", sim.i, "\\en and lars\\lars.Rdata", sep=""))
	beta.lars = beta.lars[1:50,]
	if (length(ind0)==0) {type1.lars=0} else {type1.lars=(1 - apply(beta.lars[,ind0] == 0, 1, sum) / length(ind0))}
	type2.lars = apply(beta.lars[,ind1] == 0, 1, sum) / length(ind1)

	for (method in c(1,3)) {
		type1.ben = type2.ben = type1.bl = type2.bl = NULL
		for (d in seq(0.0001,1,length=length)) {
			source("result process.R")
			# false inclusion
			if (length(ind0)==0) {
				type1.ben = cbind(type1.ben, 0)
			} else {type1.ben = cbind(type1.ben, (1 - apply(bsm.ben[,ind0], 1, sum) / length(ind0)))}
			# false exclusion
			type2.ben = cbind(type2.ben, apply(bsm.ben[,ind1], 1, sum) / length(ind1))
			if (length(ind0)==0) {
				type1.bl = cbind(type1.bl, 0)
			} else {type1.bl = cbind(type1.bl, (1 - apply(bsm.bl[,ind0], 1, sum) / length(ind0)))}
			type2.bl = cbind(type2.bl, apply(bsm.bl[,ind1], 1, sum) / length(ind1))
		}
		temp1 = data.frame(type1.ben, type=1,model="ben",method=method)
		temp2 = data.frame(type2.ben, type=2,model="ben",method=method)
		temp3 = data.frame(type1.bl, type=1,model="bl",method=method)
		temp4 = data.frame(type2.bl, type=2,model="bl",method=method)
		error = rbind(error, temp1,temp2,temp3,temp4)
	}

#---	For simulation 1 and 3, we make both plots. And for simulation 2, we only make the second plot.

	method = 1
	plot(	apply(error[(error$type==1&error$model=="ben"&error$method==method),1:length],2,mean), 
		1 - apply(error[(error$type==2&error$model=="ben"&error$method==method),1:length],2,mean), 
		type="l", 
		ylim=c(0,1),
		xlab="the false inclusion rate", ylab="the correct inclusion rate",
		main=paste("Simulation ",sim.i,": ROC curve",sep=""),
		lwd=2)
	lines(	apply(error[(error$type==2&error$model=="bl"&error$method==method),1:length],2,mean), 
		1 - apply(error[(error$type==1&error$model=="bl"&error$method==method),1:length],2,mean), lty="dotted",
		lwd=2)
	method = 3
	lines(	apply(error[(error$type==2&error$model=="ben"&error$method==method),1:length],2,mean), 
		1 - apply(error[(error$type==1&error$model=="ben"&error$method==method),1:length],2,mean), lty="dotdash",
		lwd=2)
	lines(	apply(error[(error$type==2&error$model=="bl"&error$method==method),1:length],2,mean), 
		1 - apply(error[(error$type==1&error$model=="bl"&error$method==method),1:length],2,mean), lty="dashed",
		lwd=2)
	points(mean(type1.en),1-mean(type2.en),pch=3,lwd=3)
	points(mean(type1.lars),1-mean(type2.lars),pch=4,lwd=3)
	abline(v = 0.5)
	legend(	locator(n=1), legend = c("BEN+CI","BL+CI", "BEN+SN", "BL+SN"), 
		lty=c("solid","dotted","dotdash","dashed"), lwd=c(2,2,2,2), bty="n")
	savePlot(filename=paste("VS_sim",sim.i,"_ROC",sep=""),type="eps")

	method = 1
	plot(	seq(0.0001,1,length=length),
		1 - apply(error[(error$type==2&error$model=="ben"&error$method==method),1:length],2,mean), 
		type="l", 
		xlab="the threshold value", ylab="the correct inclusion rate",
		ylim=c(0,1),
		main=paste("Simulation ",sim.i,": power curve",sep=""),
		lwd=2)
	lines(	seq(0.0001,1,length=length),
		1 - apply(error[(error$type==2&error$model=="bl"&error$method==method),1:length],2,mean), lty="dotted",
		lwd=2)
	method = 3
	lines(	seq(0.0001,1,length=length),
		1 - apply(error[(error$type==2&error$model=="ben"&error$method==method),1:length],2,mean), lty="dotdash",
		lwd=2)
	lines(	seq(0.0001,1,length=length),
		1 - apply(error[(error$type==2&error$model=="bl"&error$method==method),1:length],2,mean), lty="dashed",
		lwd=2)
	abline(h=1-mean(type2.en),lwd=1)
	abline(h=1-mean(type2.lars),lwd=1,lty="longdash")
	abline(v = 0.5)
	legend(	locator(n=1), legend = c("BEN+CI","BL+CI", "BEN+SN", "BL+SN"), 
		lty=c("solid","dotted","dotdash","dashed"), lwd=c(2,2,2,2), bty="n")
	savePlot(filename=paste("VS_sim",sim.i,"_power",sep=""),type="eps")
