	data(Prostate)
	x = as.matrix(Prostate[,-9])
	y = Prostate[,9]
	x.train = x[1:67,]
	x.test = x[-(1:67),]
	y.train = y[1:67]
	y.test = y[-(1:67)]

	#---	Using BL

	lambda.ini = GSIN(scale(x),y-mean(y))
	result.bl = GSL_BL(x,y,lambda1=lambda.ini$lambda1, tol=0.0001, itermax=20)

	#---	Using BEN

	lambda.ini = GSIN(scale(x.train),y.train-mean(y.train))
	result.ben = GSL(x.train,y.train,lambda1=lambda.ini$lambda1,lambda2=1, itermax=30, tol=0.005)

	#---	Using ENET

	#	result.en = ENL(scale(x.train),y.train - mean(y.train),lambda2 = c(0,0.01,0.1,1, 5, 10, 30, 40, 50,100,1000))
	
	#	For some reason, the cross-validation I did here does not pick the same penalty parameter as 
	#	those in the paper. So to match the results in the paper, I would use the parameters provided
	#	by the paper.
	
	temp = enet(scale(x.train),y.train-mean(y.train),lambda=1000)
	result.en = list(beta = predict(temp, s = 0.26, type = "coefficients", mode = "fraction")$coef, s = 0.26, lambda2 = 1000)

	#---	Using LARS

	#	result.lars = LARSL(scale(x.train),y.train-mean(y.train))	
	#	For some reason, the cross-validation I did here does not pick the same penalty parameter as 
	#	those in the paper. So to match the results in the paper, I would use the parameters provided
	#	by the enet paper.

	temp = lars(x = scale(x.train), y = y.train-mean(y.train))
	result.lars = list(beta = predict(temp, s = 0.39, type = "coefficients", mode = "fraction")$coef, s = 0.39)
	
	#---	Using Least Square

	result.ls = lm(y.train-mean(y.train) ~ 0 + scale(x.train))

	save.image(file = "BENProstate.RData")

#---	3.1. Load the workspace

	load("dia and pro\\BENProstate.RData")

#---	3.2. Some plots

	for (i in 1:8) {
		if((i-1)/4 == as.integer(i/4)) {
			x11()
			par(mfrow = c(2,2))
		}

		temp = result.ben$beta[,i]
		plot(density(temp),main=i)
		lines(sort(temp),dnorm(sort(temp),mean=mean(temp), sd=sqrt(var(temp))),col="red")
		temp.line = quantile(temp, probs = c(0.025, 0.975))
		abline(v = temp.line)

		#plot(result.ben$beta[,i], type="l")
		#acf(result.ben$beta[,i])
		#hist(result.ben$beta[,i])
	}
	
	plot(seq(-0.3,0.7,length=8), 1:8, ylim=rev(range(1:8)), axes=F, type="n", xlab="Coefficients", ylab="Variable number")
	axis(1, at = seq(-0.3,0.7,by=0.2))
	axis(2, at = seq(1, 8, 1))
	box()
	abline(v=0)
	temp1 = apply(result.ben$beta, 2, median)
	temp2 = apply(result.bl$beta, 2, median)
	for (i in 1:8) {
		points(result.ls$coef[i],i,pch=4)
		points(temp1[i], i, pch=3)
		points(temp2[i], i, pch=1)
		points(result.en$beta[i],i+0.15,pch=2)
		points(result.lars$beta[i],i-0.15,pch=6)
	}
 
#---	3.3. The mean-squred error

	#	x.test.bar = (x.test - matrix(rep(apply(x.train, 2, mean), dim(x.test)[1]), ncol=8, byrow=T)) %*% (1/sqrt(diag(var(x.train))))
	#	y.test.bar = y.test - mean(y.train)
	x.test.bar = scale(x.test)
	y.test.bar = y.test - mean(y.test)
	mean((y.test.bar - x.test.bar %*% apply(result.ben$beta, 2, median))^2)
	mean((y.test.bar - x.test.bar %*% apply(result.bl$beta, 2, median))^2)
	mean((y.test.bar - x.test.bar %*% result.en$beta)^2)
	mean((y.test.bar - x.test.bar %*% result.lars$beta)^2)
	mean((y.test.bar - x.test.bar %*% result.ls$coef)^2)

#---	3.4. Variable selection
	
	level = 0.5
	method = 3
	names = c("lcavol","lweight","age","lbph","svi","lcp","gleason","pgg45") 
	d = 0.5

	intervals = NULL
	interval.prob = NULL
	for (i in 1:8) {
		if (method == 1) intervals = rbind(intervals, quantile(result.ben$beta[,i], probs = c(level/2, 1-level/2)))
		if (method == 2) intervals = rbind(intervals, c(-1,1) * d)
		if (method == 3) intervals = rbind(intervals, c(-1,1) * sqrt(var(result.ben$beta[,i])))
	}
	for (i in 1:8) {
		temp = ecdf(result.ben$beta[,i])
		interval.prob = c(interval.prob, temp(intervals[i,2]) - temp(intervals[i,1]))
	}
	vs.ben = data.frame(names, intervals, interval.prob) 

	intervals = NULL
	interval.prob = NULL
	for (i in 1:8) {
		if (method == 1) intervals = rbind(intervals, quantile(result.bl$beta[,i], probs = c(level/2, 1-level/2)))
		if (method == 2) intervals = rbind(intervals, c(-1,1) * d)
		if (method == 3) intervals = rbind(intervals, c(-1,1) * sqrt(var(result.bl$beta[,i])))
	}
	for (i in 1:8) {
		temp = ecdf(result.bl$beta[,i])
		interval.prob = c(interval.prob, temp(intervals[i,2]) - temp(intervals[i,1]))
	}
	vs.bl = data.frame(names, intervals, interval.prob) 
